

import unittest
from lawes.test.test_db_models_fields import *
from lawes.test.test_db_models_base import *
from lawes.test.test_db_models_query_utils_q import *
from lawes.test.all_doctest import all_doctest

if __name__ == "__main__":
    all_doctest()
    unittest.main()